@@ -16,14 +16,17 @@ from audeer.core.tqdm import (
 )
 from audeer.core.utils import (
     deprecated,
+    deprecated_default_value,
     deprecated_keyword_argument,
     flatten_list,
     freeze_requirements,
     git_repo_tags,
     git_repo_version,
+    is_semantic_version,
     is_uid,
     run_tasks,
     run_worker_threads,
+    sort_versions,
     to_list,
     uid,
 )
