@@ -1,8 +1,10 @@
 import concurrent.futures
 from collections.abc import Iterable
 import copy
+from distutils.version import LooseVersion
 import functools
 import hashlib
+import inspect
 import queue
 import threading
 import typing
@@ -64,6 +66,54 @@ def deprecated(
     return _deprecated
 
 
+def deprecated_default_value(
+        *,
+        argument: str,
+        change_in_version: str,
+        new_default_value: typing.Any,
+) -> typing.Callable:
+    """Mark default value of keyword argument as deprecated.
+
+    Provide a `decorator <https://www.python.org/dev/peps/pep-0318/>`_
+    to mark the default value of a keyword argument as deprecated.
+    You have to specify the version
+    for which the default value will change
+    and the new default value.
+
+    Args:
+        argument: keyword argument
+        change_in_version: version the default value will change
+        new_default_value: new default value
+
+    Example:
+        >>> @deprecated_default_value(
+        ...     argument='foo',
+        ...     change_in_version='2.0.0',
+        ...     new_default_value='bar',
+        ... )
+        ... def deprecated_function(foo='foo'):
+        ...     pass
+    """
+    def _deprecated(func):
+        # functools.wraps preserves the name
+        # and docstring of the decorated code:
+        # https://docs.python.org/3/library/functools.html#functools.wraps
+        @functools.wraps(func)
+        def new_func(*args, **kwargs):
+            if argument not in kwargs:
+                signature = inspect.signature(func)
+                default_value = signature.parameters[argument].default
+                message = (
+                    f"The default of '{argument}' will change from "
+                    f"'{default_value}' to '{new_default_value}' "
+                    f'with version {change_in_version}.'
+                )
+                warnings.warn(message, category=UserWarning, stacklevel=2)
+            return func(*args, **kwargs)
+        return new_func
+    return _deprecated
+
+
 def deprecated_keyword_argument(
         *,
         deprecated_argument: str,
@@ -257,6 +307,64 @@ def git_repo_version(
     return version
 
 
+def is_semantic_version(version: str) -> bool:
+    r"""Check if given string represents a `semantic version`_.
+
+    Your version has to comply to ``X.Y.Z`` or ``vX.Y.Z``,
+    where X, Y, Z are all integers.
+    Additional version information, like ``beta``
+    has to be added using a ``-`` or ``+``,
+    e.g. ``X.Y.Z-beta``.
+
+    .. _semantic version: https://semver.org
+
+    Args:
+        version: version string
+
+    Returns:
+        ``True`` if version is a semantic version
+
+    Example:
+        >>> is_semantic_version('v1')
+        False
+        >>> is_semantic_version('1.2.3-r3')
+        True
+        >>> is_semantic_version('v0.7.2-9-g1572b37')
+        True
+
+    """
+    version_parts = version.split('.')
+
+    if len(version_parts) < 3:
+        return False
+
+    def is_integer_convertable(x):
+        try:
+            int(x)
+            return True
+        except ValueError:
+            return False
+
+    x, y = version_parts[:2]
+    # Ignore starting 'v'
+    if x.startswith('v'):
+        x = x[1:]
+
+    z = '.'.join(version_parts[2:])
+    # For Z, '-' and '+' are also allowed as separators,
+    # but you are not allowed to have an additonal '.' before
+    z = z.split('-')[0]
+    z = z.split('+')[0]
+    if len(z.split('.')) > 1:
+        return False
+
+    for v in (x, y, z):
+        if not is_integer_convertable(v):
+            return False
+
+    return True
+
+
 def is_uid(uid: str) -> bool:
     r"""Check if string is a unique identifier.
 
@@ -466,6 +574,52 @@ def run_worker_threads(
     return results
 
 
+def sort_versions(
+        versions: List[str],
+) -> List:
+    """Sort version numbers.
+
+    If a version starts with ``v``,
+    the ``v`` is ignored during sorting.
+
+    Args:
+        versions: sequence with semantic version numbers
+
+    Returns:
+        sorted list of versions with highest as last entry
+
+    Raises:
+        ValueError: if the version does not comply
+            with :func:`is_semantic_version`
+
+    Example:
+        >>> vers = [
+        ...     '2.0.0',
+        ...     '2.0.1',
+        ...     'v1.0.0',
+        ...     'v2.0.0-1-gdf29c4a',
+        ... ]
+        >>> sort_versions(vers)
+        ['v1.0.0', '2.0.0', 'v2.0.0-1-gdf29c4a', '2.0.1']
+
+    """
+    for version in versions:
+        if not is_semantic_version(version):
+            raise ValueError(
+                "All version numbers have to be semantic versions, "
+                "following 'X.Y.Z', "
+                "where X, Y, Z are integers. "
+                f"But your version is: '{version}'."
+            )
+
+    def sort_key(value):
+        if value.startswith('v'):
+            value = value[1:]
+        return LooseVersion(value)
+
+    return sorted(versions, key=sort_key)
+
+
 def to_list(x: Any):
     """Convert to list.
 
